/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.area.field;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Random;
import nl.cloudfarming.client.geoviewer.LayerObject;
import nl.cloudfarming.client.geoviewer.StatePalette;
import nl.cloudfarming.client.model.PartField;
import nl.cloudfarming.client.model.Race;
import org.netbeans.api.visual.model.ObjectState;

/**
 * Color palette for partField
 * Provides colors for the PartField object
 * @author Timon Veenstra
 */
public class PartFieldPalette extends StatePalette {

    private final Color colorSelected = Color.RED;
    private final Color colorHoovered = Color.BLUE;
    private static final String OBJECT_NAME = "partField";
    private static HashMap<Race, Color> raceColorMap;
    private static ArrayList<Color> availableColors = new ArrayList<Color>();

    /**
     * Add a bunch of predefined colors to hour list
     */
    public PartFieldPalette() {
        availableColors.add(Color.YELLOW);
        availableColors.add(Color.RED);
        availableColors.add(Color.BLUE);
        availableColors.add(Color.GREEN);
        availableColors.add(Color.ORANGE);
        availableColors.add(Color.PINK);
        availableColors.add(Color.GRAY);
        availableColors.add(Color.CYAN);
        availableColors.add(Color.MAGENTA);
    }

    @Override
    public Color getColorForState(final ObjectState state, LayerObject layerObject) {
        if (state.isSelected()) {
            return colorSelected;
        }
        if (state.isHovered() || state.isObjectHovered()) {
            return colorHoovered;
        }
        return layerObject.getObjectColor();
    }

    @Override
    public String getSensorName() {
        return OBJECT_NAME;
    }

    @Override
    public Class forClass() {
        return PartField.class;
    }

    @Override
    public List getPaletteOptions() {
        return Collections.EMPTY_LIST;
    }

    /**
     * Gets a color to represent a Crop Race.
     * First it will get the color from a predefined list of 9 colors.
     * if this list is exhausted a random color will be returned.
     * the random color can never be the same as the predefined colors
     * @param race the race for which the color is selected.
     * @return Unique Color
     */
    public static Color getColorForRace(Race race) {
        if (raceColorMap == null) {
            raceColorMap = new HashMap<Race, Color>();
        }
        if (raceColorMap.containsKey(race)) {
            return raceColorMap.get(race);
        } else {
            if (!availableColors.isEmpty()) {
                Color returnColor = availableColors.get(0);
                availableColors.remove(returnColor);
                raceColorMap.put(race, returnColor);
                return returnColor;
            } else {
                return getRandomColor(race);
            }
        }
    }

    /**
     * Gets a random Color.
     * Will check the already distributed Colors and never return the same color.
     * @param race the Race for which a color is selected
     * @return the selected Color
     */
    private static Color getRandomColor(Race race) {
        Color returnColor = RandomColor.randomColor();
        for (Race mapRace : raceColorMap.keySet()) {
            if (raceColorMap.get(mapRace).equals(returnColor)) {
                getRandomColor(race);
                break;
            }
        }
        return returnColor;
    }

    /**
     * Class to generate Random Colors
     */
    private static class RandomColor {

        private static Random rand;

        /**
         * Constructor for objects of class RandomColor initializes the
         * random number generator
         */
        public RandomColor() {
            rand = new Random();
        }

        /**
         * randomColor returns a random Color
         * 
         * @return a random Color
         */
        public static Color randomColor() {
            return (new Color(rand.nextInt(256),
                    rand.nextInt(256),
                    rand.nextInt(256)));
        }
    }
}
